'use strict';

const _ = require('lodash');
const assert = require('assert');
const debug = require('debug')('koa-raven');
const Raven = require('raven');

module.exports = function (DSN, opts) {
  assert('string' === typeof DSN, 'DSN required');
  opts = opts || {};
  opts.environment = opts.environment || process.env.NODE_ENV;

  const raven = new Raven.Client(DSN, opts);

  process.on('uncaughtException', function (e) {
    console.error('uncaughtException: ', e);
    raven.captureException(e, {
      level: 'fatal'
    });
    setTimeout(function () {
      process.exit(1);
    }, 100);
  });
  function captureException(ctx, status, e) {
    if (status >= 400 && status < 500) {
      raven.captureException(e, {
        level: 'warning',
        extra: {
          this: {
            request: _.pick(ctx.request, 'url', 'method', 'header', 'body', 'query', 'ips'),
            response: ctx.response
          }
        }
      });
    } else if (status >= 500) {
      raven.captureException(e, {
        level: 'error',
        extra: {
          this: {
            request: _.pick(ctx.request, 'url', 'method', 'header', 'body', 'query', 'ips'),
            response: ctx.response
          }
        }
      });
    }
  }

  return function* koaRaven(next) {
    this.raven = raven;
    try {
      yield next;
    } catch (e) {
      debug(e);
      captureException(this, e.status || e.statusCode || this.status || 500, e);
      throw e;
    }
    captureException(this, this.status, this.body && this.body.error);
  };
};
